from pysnmp.smi.macro import rfc2578
from pysnmp.proto import rfc1902

( ConstraintsIntersection, ConstraintsUnion, SingleValueConstraint,
  ValueRangeConstraint, ValueSizeConstraint, ) = mibBuilder.importSymbols(
    "ASN1-REFINEMENT", "ConstraintsIntersection", "ConstraintsUnion",
    "SingleValueConstraint", "ValueRangeConstraint", "ValueSizeConstraint"
    )

class ExtUTCTime(rfc1902.OctetString):
    subtypeSpec = rfc1902.OctetString.subtypeSpec+ConstraintsUnion(ValueSizeConstraint(11,11), ValueSizeConstraint(13,13))

zeroDotZero = rfc2578.ObjectIdentity((0,0))

#dot = MibTree()
iso = rfc2578.MibTree((1,))
org = rfc2578.MibIdentifier(iso.name + (3,))
dod = rfc2578.MibIdentifier(org.name + (6,))
internet = rfc2578.MibIdentifier(dod.name + (1,))
directory = rfc2578.MibIdentifier(internet.name + (1,))
mgmt = rfc2578.MibIdentifier(internet.name + (2,))
mib_2 = rfc2578.MibIdentifier(mgmt.name + (1,)).setLabel('mib-2')
transmission = rfc2578.MibIdentifier(mib_2.name + (10,))
experimental = rfc2578.MibIdentifier(internet.name + (3,))
private = rfc2578.MibIdentifier(internet.name + (4,))
enterprises = rfc2578.MibIdentifier(private.name + (1,))
security = rfc2578.MibIdentifier(internet.name + (5,))
snmpV2 = rfc2578.MibIdentifier(internet.name + (6,))

snmpDomains = rfc2578.MibIdentifier(snmpV2.name + (1,))
snmpProxys = rfc2578.MibIdentifier(snmpV2.name +(2,))
snmpModules = rfc2578.MibIdentifier(snmpV2.name +(3,))

mibBuilder.exportSymbols(
    'SNMPv2-SMI',
    # syntax of objects
    OctetString=rfc1902.OctetString,
    Bits=rfc1902.Bits,
    Integer32=rfc1902.Integer32,
    IpAddress=rfc1902.IpAddress,
    Counter32=rfc1902.Counter32,
    Gauge32=rfc1902.Gauge32,
    Unsigned32=rfc1902.Unsigned32,
    TimeTicks=rfc1902.TimeTicks,
    Opaque=rfc1902.Opaque,
    Counter64=rfc1902.Counter64,
    # 
    ExtUTCTime=ExtUTCTime,
    # SMIv2 macros implementations
    ModuleIdentity=rfc2578.ModuleIdentity,
    ObjectIdentity=rfc2578.ObjectIdentity,
    ObjectType=rfc2578.ObjectType,
    NotificationType=rfc2578.NotificationType,
    # Concise MIB node types
    MibIdentifier=rfc2578.MibIdentifier,
    MibScalar=rfc2578.MibScalar,
    MibScalarInstance=rfc2578.MibScalarInstance,
    MibTableColumn=rfc2578.MibTableColumn,
    MibTableRow=rfc2578.MibTableRow,
    MibTable=rfc2578.MibTable,
    MibTree=rfc2578.MibTree,
    # Initial MIB tree identifiers
    zeroDotZero=zeroDotZero,
    iso=iso, org=org, dod=dod, internet=internet,
    directory=directory, mgmt=mgmt, mib_2=mib_2, transmission=transmission,
    experimental=experimental, private=private, enterprises=enterprises,
    security=security, snmpV2=snmpV2, snmpDomains=snmpDomains,
    snmpProxys=snmpProxys, snmpModules=snmpModules    
    )

# XXX
# getAsName/setFromName goes out of MibRow?
# revisit getNextNode() -- needs optimization
